@@ -2,10 +2,13 @@
 # Copyright (c) 2020 Arista Networks, Inc.  All rights reserved.
 # Arista Networks, Inc. Confidential and Proprietary.
 
-__version__ = "0.6.0"
+__version__ = "0.6.2"
 
 # from eapi.constants import ENCODING, INCLUDE_TIMESTAMPS, SSL_VERIFY, \
 #                          SSL_WARNINGS, TIMEOUT
 
-from eapi.sessions import Session
-from eapi.api import configure, enable, execute, watch
\ No newline at end of file
+import eapi.environments
+import eapi.types
+
+from eapi.sessions import Session, AsyncSession
+from eapi.api import aexecute, awatch, configure, enable, execute, watch
\ No newline at end of file
